package cn.zhangdeman;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.util.Map;

// 初始化的请求信息
@Getter
@Setter
public class InitRequestData implements Serializable {
    @JsonProperty(RecordField.REQUEST_ID)
    private String requestId; // 本次请求request id
    @JsonProperty(RecordField.COST)
    private Long cost; // 请求耗时
    @JsonProperty(RecordField.SERVER_IP)
    private String serverIp; // 服务器Ip
    @JsonProperty(RecordField.SERVER_HOSTNAME)
    private String serverHostname; // 服务器hostname
    @JsonProperty(RecordField.REQUEST_UA)
    private String userAgent; // 客户端ua
    @JsonProperty(RecordField.REQUEST_CLIENT_IP)
    private String clientIp; // 客户端IP
    @JsonProperty(RecordField.FINISH_TIMESTAMP)
    private Long finishTimeStamp; // 完成请求时间
    @JsonProperty(RecordField.TRACE_ID)
    private String traceId; // 全局trace id
    @JsonProperty(RecordField.START_TIMESTAMP)
    private Long startTimeStamp; // 开始请求时间
    @JsonProperty(RecordField.CONTEXT_DATA)
    private Map<String, Object> logData; // 本条日志的上下文信息
    @JsonProperty(RecordField.RESPONSE_DATA)
    private Object responseData; // 响应数据

    // 序列化
    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    // 序列化
    public String serializable() {
        return toString();
    }

    // 反序列化
    public InitRequestData unserializable(String json) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        return mapper.readValue(json, new TypeReference<InitRequestData>() {
        });
    }
}
